package
{
	import com.greensock.TweenMax;
	import com.greensock.easing.Linear;
	
	import flashx.textLayout.formats.Float;
	
	import mx.utils.StringUtil;
	
	import org.flixel.FlxG;
	import org.flixel.FlxObject;
	import org.flixel.FlxPath;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;

	public class Game
	{
		//>1 = passable	
		public var map:Vector.<Vector.<int>>;
		public var blocks:Vector.<BlockDat>;
		public var player:EntDat;
		public var exit:EntDat;
		
		public function MapString():String
		{
			var s:String="";
			for(var i:int=0;i<map.length;i++)
			{
				if (i>0)
					s+="\n";
				
				for (var j:int=0;j<map[i].length;j++)
				{
					if (j>0)
						s+=",";
					
					s+=map[i][j];
					
				}
			}
			return s;
		}			
				
		public function ClearMoved():void
		{
			for each(var ed:BlockDat in blocks)
			{
				ed.moved=FlxObject.NODIR;
				ed.trymoved=false;
			}
			player.moved=FlxObject.NODIR;
			player.trymoved=false;
			exit.moved=FlxObject.NODIR;
			exit.trymoved=false;
		}
		
		function Game(desc:String="")
		{
			map = new Vector.<Vector.<int>>();
			blocks = new Vector.<BlockDat>();
			player = new EntDat(-1,-1);
			exit = new EntDat(-1,-1);
			
			if (desc=="")
				return;
			
			var ar:Array = desc.split("\n");
			for (var i:int=0;i<ar.length;i++)
			{
				var row:Vector.<int> = new Vector.<int>();
				var s:String = ar[i];				
				//s = mx.utils.StringUtil.trim(s);
				for (var j:int=0;j<s.length;j++)
				{
					switch(s.charAt(j))
					{
						case "#":
							row.push(1);
							break;
						case " ":
							row.push(2);
							break;
						case ".":
							row.push(3);
							break;
						case "a":
							row.push(2);
							blocks.push(new BlockDat(j,i,false,false));
							break;
						case "@":
							row.push(3);
							blocks.push(new BlockDat(j,i,false,false));
							break;
						case "P":
							player = new EntDat(j,i);
							row.push(2);
							break;
						case "B":
							player = new EntDat(j,i);
							row.push(3);
							break;
					}
				}
				map.push(row);
			}
			CheckWin();
		}
		
		private function TileMapBlocks(x:int,y:int):Boolean
		{
			if (x<0 || y<0 || x>=map[0].length || y>=map.length)
				return true;
			return map[y][x]<=1;
		}
		
		private function GetBlock(x:int,y:int):BlockDat
		{
			for each (var bd:BlockDat in blocks)
			{
				if (bd.x==x&&bd.y==y)
					return bd;
			}
			return null;
		}
		
		public function TryMove(ed:EntDat, dir:uint):Boolean
		{		
			if (ed.trymoved)
				return dir==ed.moved;
			
			var tx:int=ed.x;
			var ty:int=ed.y;
			switch(dir)
			{
				case FlxObject.UP:
					ty--;
					break;
				case FlxObject.DOWN:
					ty++;
					break;
				case FlxObject.LEFT:
					tx--;
					break;
				case FlxObject.RIGHT:
					tx++;
					break;
			}
			
			var tilemapblocked:Boolean = TileMapBlocks(tx,ty);
			if (tilemapblocked==true)
			{
				return false;
			}
			
			if (tx<0 || tx>=map[0].length || ty<0 || ty>=map.length)
			{
				return false;
			}
			
			var bd:BlockDat = GetBlock(tx,ty);
			
			if (bd!=null)
			{
				if (TryMove(bd,dir)==false)
					return false;
			}			
						
			//success! do stuff!
			
			
			ed.moved=dir;
			
			ed.trymoved=true;
			
			return true;
			
	
		}
		
		public function ProcessMoves():Boolean
		{
			ProcessMove(player);
			for each(var bd:BlockDat in blocks)
			{
				ProcessMove(bd);
			}
			
			return CheckWin();
		}
		
		private function CheckWin():Boolean
		{			
			for each(var bd:BlockDat in blocks)
			{
				bd.satisfied=map[bd.y][bd.x]==3;
				
				if (bd.satisfied ==false)
					return false;
			}
			return true;
		}
		
		private function ProcessMove(e:EntDat):void
		{
			switch(e.moved)
			{
				case FlxObject.NODIR:
					break;
				case FlxObject.UP:
					e.y--;
					break;
				case FlxObject.DOWN:
					e.y++;
					break;
				case FlxObject.LEFT:
					e.x--;
					break;
				case FlxObject.RIGHT:
					e.x++;
					break;
			}
		}
		
		public var movedur:Number = 0.2;		
			
		
		public function Copy():Game
		{
			var g:Game = new Game();
			for (var i:int=0;i<map.length;i++)
			{
				var r:Vector.<int> = new Vector.<int>();
				for (var j:int=0;j<map[i].length;j++)
				{
					r.push(map[i][j]);
				}
				g.map.push(r);
			}
			
			for (i=0;i<blocks.length;i++)
			{
				g.blocks.push(blocks[i].Copy());				
			}
			
			g.exit=exit.Copy();
			g.player=player.Copy();
			
			return g;
		}
	}
}